/* HPCU Bi-Weekly Challenge: MPI Pi - starter code
 * Author: Aaron Weeden, Shodor, May 2015
 *
 * Approximate pi using a Left Riemann Sum under a quarter unit circle.
 *
 * When running the program, the number of rectangles can be passed using the
 * -r option, e.g. 'pi-serial -r X', where X is the number of rectangles.
 */

/*************
 * LIBRARIES *
 *************/
#include <stdio.h>  /* printf() */
#include <stdlib.h> /* exit(), EXIT_FAILURE */
#include <math.h>   /* sqrt() */
#include <float.h>  /* FLT_EPSILON and LDBL_DIG */
#include <unistd.h> /* getopt() */

/*************************
 * FUNCTION DECLARATIONS *
 *************************/
void getUserOptions(int argc, char **argv, int *numRects);

/************************
 * FUNCTION DEFINITIONS *
 ************************/
int main(int argc, char **argv) {
  int numRects = 10;
  float width = 0.0;
  float x = 0.0;
  float heightSq = 0.0;
  float height = 0.0;
  float area = 0.0;
  float pi = 0.0;
  int i = 0;

  getUserOptions(argc, argv, &numRects);

  width = (1.0 / numRects);

  for (i = 0; i < numRects; i++) {
    x = (i * width);

    heightSq = (1.0 - (x * x));

    /* Prevent nan value for sqrt() */
    if (heightSq < FLT_EPSILON) {
      heightSq = 0.0;
    }

    height = sqrt(heightSq);

    area += (width * height);
  }

  pi = (4.0 * area);

  printf("%.*f\n", LDBL_DIG, pi);

  return 0;
}

void getUserOptions(int argc, char **argv, int *numRects) {
  char c;

  while ((c = getopt(argc, argv, "r:")) != -1) {
    switch(c) {
      case 'r':
        (*numRects) = atoi(optarg);
        break;
      case '?':
      default:
        fprintf(stderr, "Usage: ");
        fprintf(stderr, "%s [-r numRects]\n", argv[0]);
        exit(EXIT_FAILURE);
    }
  }
  argc -= optind;
  argv += optind;
}
